@@ -55,14 +55,11 @@
 #define MAC "%02x:%02x:%02x:%02x:%02x:%02x"
 #define MAC_STR(a) a[0], a[1], a[2], a[3], a[4], a[5]
 
-#define IP_STR(uint_ip) \
-({ \
-	struct in_addr _in; \
-	char *_out; \
-	_in.s_addr = uint_ip; \
-	_out = inet_ntoa(_in); \
-	_out; \
-})
+#define NIPQUAD_FMT "%u.%u.%u.%u"
+#define NIPQUAD(u32_ip)	((unsigned char *) &u32_ip)[0], \
+			((unsigned char *) &u32_ip)[1], \
+			((unsigned char *) &u32_ip)[2], \
+			((unsigned char *) &u32_ip)[3]
 
 #define ACD_DEBUG(fmt, args...)					\
 	l_util_debug(acd->debug_handler, acd->debug_data,		\
@@ -146,7 +143,8 @@ static int acd_send_packet(struct l_acd *acd, uint32_t source_ip)
 	p.arp_pln = 4;
 	p.arp_op = htons(ARPOP_REQUEST);
 
-	ACD_DEBUG("sending packet with target IP %s", IP_STR(acd->ip));
+	ACD_DEBUG("sending packet with target IP "NIPQUAD_FMT,
+			NIPQUAD(acd->ip));
 
 	memcpy(&p.arp_sha, acd->mac, ETH_ALEN);
 	memcpy(&p.arp_spa, &source_ip, sizeof(p.arp_spa));
@@ -165,8 +163,8 @@ static void announce_wait_timeout(struct l_timeout *timeout, void *user_data)
 	struct l_acd *acd = user_data;
 
 	if (acd->state == ACD_STATE_PROBE) {
-		ACD_DEBUG("No conflicts found for %s, announcing address",
-				IP_STR(acd->ip));
+		ACD_DEBUG("No conflicts found for "NIPQUAD_FMT ", announcing address",
+				NIPQUAD(acd->ip));
 
 		acd->state = ACD_STATE_ANNOUNCED;
 
@@ -183,10 +181,13 @@ static void announce_wait_timeout(struct l_timeout *timeout, void *user_data)
 	}
 
 	if (acd->retries != ANNOUNCE_NUM) {
+		int err = acd_send_packet(acd, acd->ip);
+
 		acd->retries++;
 
-		if (acd_send_packet(acd, acd->ip) < 0) {
-			ACD_DEBUG("Failed to send ACD announcement");
+		if (err < 0) {
+			ACD_DEBUG("Failed to send ACD announcement: %s",
+					strerror(-err));
 			return;
 		}
 
@@ -204,12 +205,13 @@ static void announce_wait_timeout(struct l_timeout *timeout, void *user_data)
 static void probe_wait_timeout(struct l_timeout *timeout, void *user_data)
 {
 	struct l_acd *acd = user_data;
+	int err;
 	uint32_t delay;
 
 	ACD_DEBUG("Sending ACD Probe");
 
-	if (acd_send_packet(acd, 0) < 0) {
-		ACD_DEBUG("Failed to send ACD probe");
+	if ((err = acd_send_packet(acd, 0)) < 0) {
+		ACD_DEBUG("Failed to send ACD probe: %s", strerror(-err));
 		return;
 	}
 
@@ -262,6 +264,7 @@ static bool acd_read_handler(struct l_io *io, void *user_data)
 	int source_conflict;
 	int target_conflict;
 	bool probe;
+	int err;
 
 	memset(&arp, 0, sizeof(arp));
 	len = read(l_io_get_fd(acd->io), &arp, sizeof(arp));
@@ -283,18 +286,15 @@ static bool acd_read_handler(struct l_io *io, void *user_data)
 	target_conflict = probe &&
 		!memcmp(arp.arp_tpa, &acd->ip, sizeof(uint32_t));
 
-	if (!source_conflict && !target_conflict) {
-		ACD_DEBUG("No target or source conflict detected for %s",
-				IP_STR(acd->ip));
+	if (!source_conflict && !target_conflict)
 		return true;
-	}
 
 	switch (acd->state) {
 	case ACD_STATE_PROBE:
 		/* No reason to continue probing */
-		ACD_DEBUG("%s conflict detected for %s",
+		ACD_DEBUG("%s conflict detected for "NIPQUAD_FMT,
 				target_conflict ? "Target" : "Source",
-				IP_STR(acd->ip));
+				NIPQUAD(acd->ip));
 
 		if (acd->event_func)
 			acd->event_func(L_ACD_EVENT_CONFLICT, acd->user_data);
@@ -317,12 +317,12 @@ static bool acd_read_handler(struct l_io *io, void *user_data)
 		if (acd->policy == L_ACD_DEFEND_POLICY_NONE) {
 			ACD_DEBUG("Conflict detected, giving up address");
 
+			l_acd_stop(acd);
+
 			if (acd->event_func)
 				acd->event_func(L_ACD_EVENT_LOST,
 							acd->user_data);
 
-			l_acd_stop(acd);
-
 			break;
 		}
 
@@ -345,7 +345,10 @@ static bool acd_read_handler(struct l_io *io, void *user_data)
 		if (acd->timeout)
 			l_timeout_remove(acd->timeout);
 
-		acd_send_packet(acd, acd->ip);
+		err = acd_send_packet(acd, acd->ip);
+		if (err < 0)
+			ACD_DEBUG("Failed to send initial announcement: %s",
+					strerror(-err));
 
 		ACD_DEBUG("Defending address");
 
@@ -384,6 +387,8 @@ static bool acd_read_handler(struct l_io *io, void *user_data)
 		acd->timeout = NULL;
 
 		ACD_DEBUG("Lost address");
+		l_acd_stop(acd);
+
 		/*
 		* RFC 5227 Section 2.4(b)
 		* "if this is not the first conflicting ARP packet the host has seen,
@@ -395,8 +400,6 @@ static bool acd_read_handler(struct l_io *io, void *user_data)
 		if (acd->event_func)
 			acd->event_func(L_ACD_EVENT_LOST, acd->user_data);
 
-		l_acd_stop(acd);
-
 		break;
 	}
 
@@ -430,12 +433,13 @@ LIB_EXPORT bool l_acd_start(struct l_acd *acd, const char *ip)
 		return false;
 
 	if (l_memeqzero(acd->mac, ETH_ALEN) &&
-			!l_net_get_mac_address(acd->ifindex, acd->mac)) {
-		close(fd);
-		return false;
-	}
+			!l_net_get_mac_address(acd->ifindex, acd->mac))
+		goto err;
 
 	acd->io = l_io_new(fd);
+	if (!acd->io)
+		goto err;
+
 	l_io_set_close_on_destroy(acd->io, true);
 	l_io_set_read_handler(acd->io, acd_read_handler, acd, NULL);
 
@@ -473,6 +477,10 @@ LIB_EXPORT bool l_acd_start(struct l_acd *acd, const char *ip)
 							acd, NULL);
 
 	return true;
+
+err:
+	close(fd);
+	return false;
 }
 
 LIB_EXPORT bool l_acd_set_event_handler(struct l_acd *acd,
