@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2013-2016  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2013-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,7 @@
 #define _GNU_SOURCE
 #include <features.h>
 
-#ifdef HAVE_EXECINFO_H
+#ifdef HAVE_BACKTRACE
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -114,13 +114,16 @@ void __iwd_backtrace_print(unsigned int offset)
 		if (written < 0)
 			break;
 
-		len = read(infd[0], buf, sizeof(buf));
+		len = read(infd[0], buf, sizeof(buf) - 1);
 		if (len < 0)
 			break;
 
 		buf[len] = '\0';
 
 		pos = strchr(buf, '\n');
+		if (!pos)
+			continue;
+
 		*pos++ = '\0';
 
 		if (strcmp(buf, "??") == 0) {
@@ -130,6 +133,9 @@ void __iwd_backtrace_print(unsigned int offset)
 		}
 
 		ptr = strchr(pos, '\n');
+		if (!ptr)
+			continue;
+
 		*ptr++ = '\0';
 
 		if (strncmp(pos, program_path, pathlen) == 0)
@@ -156,7 +162,7 @@ static void signal_handler(int signo)
 	exit(EXIT_FAILURE);
 }
 
-void __iwd_backtrace_init()
+void __iwd_backtrace_init(void)
 {
 	static char path[PATH_MAX];
 	static char cwd[PATH_MAX];
@@ -168,6 +174,7 @@ void __iwd_backtrace_init()
 	len = readlink("/proc/self/exe", path, sizeof(path) - 1);
 	if (len > 0) {
 		int i;
+
 		path[len] = '\0';
 
 		for (i = len - 1; i >= 0; i--) {
