@@ -98,7 +98,6 @@ static int eap_md5_check_settings(struct l_settings *settings,
 	const struct eap_secret_info *secret;
 	char identity_key[72];
 	char password_key[72];
-	char password_key_old[72];
 
 	L_AUTO_FREE_VAR(char *, identity);
 	L_AUTO_FREE_VAR(char *, password) = NULL;
@@ -124,17 +123,6 @@ static int eap_md5_check_settings(struct l_settings *settings,
 	password = l_settings_get_string(settings, "Security", password_key);
 
 	if (!password) {
-		snprintf(password_key_old, sizeof(password_key_old),
-						"%sMD5-Secret", prefix);
-		password = l_settings_get_string(settings, "Security",
-							password_key_old);
-		if (password) {
-			explicit_bzero(password, strlen(password));
-			l_warn("Setting '%s' is deprecated, use '%s' instead",
-					password_key_old, password_key);
-			return 0;
-		}
-
 		secret = l_queue_find(secrets, eap_secret_info_match,
 								password_key);
 		if (secret)
