@@ -114,24 +114,24 @@ static bool erp_tlv_iter_next(struct erp_tlv_iter *iter)
 {
 	const unsigned char *tlv = iter->tlv + iter->pos;
 	const unsigned char *end = iter->tlv + iter->max;
-	unsigned int tag;
-	unsigned int len;
+	uint8_t tag;
+	uint8_t len;
 
-	if (iter->pos + 2 >= iter->max)
+	if (end - tlv < 2)
 		return false;
 
 	tag = *tlv++;
 
 	/*
-	 * These two tags are not actually TLVs (they are just type-value). Both
-	 * are 32-bit integers.
+	 * These two tags are not actually TLVs (they are just type-value).
+	 * Both are 32-bit integers.
 	 */
 	if (tag != ERP_TV_RMSK_LIFETIME && tag != ERP_TV_RRK_LIFETIME)
 		len = *tlv++;
 	else
 		len = 4;
 
-	if (tlv + len > end)
+	if (len > end - tlv)
 		return false;
 
 	iter->tag = tag;
@@ -336,6 +336,7 @@ struct erp_state *erp_new(struct erp_cache_entry *cache,
 	erp->tx_packet = tx_packet;
 	erp->user_data = user_data;
 	erp->cache = cache;
+	cache->ref++;
 
 	return erp;
 }
