@@ -88,6 +88,7 @@ struct network {
 	bool provisioning_hidden:1;
 	uint8_t transition_disable; /* Temporary cache until info is set */
 	bool have_transition_disable:1;
+	bool force_default_owe_group:1;
 	int rank;
 	/* Holds DBus Connect() message if it comes in before ANQP finishes */
 	struct l_dbus_message *connect_after_anqp;
@@ -525,6 +526,9 @@ int network_handshake_setup(struct network *network, struct scan_bss *bss,
 		handshake_state_set_protocol_version(hs, eapol_proto_version);
 	}
 
+	if (hs->akm_suite == IE_RSN_AKM_SUITE_OWE)
+		hs->force_default_owe_group = network->force_default_owe_group;
+
 	/*
 	 * The randomization options in the provisioning file are dependent on
 	 * main.conf: [General].AddressRandomization=network. Any other value
@@ -760,8 +764,20 @@ void network_set_info(struct network *network, struct network_info *info)
 		network->info = NULL;
 	}
 
+#ifdef HAVE_DBUS
 	l_dbus_property_changed(dbus_get_bus(), network_get_path(network),
 					IWD_NETWORK_INTERFACE, "KnownNetwork");
+#endif
+}
+
+void network_set_force_default_owe_group(struct network *network)
+{
+	network->force_default_owe_group = true;
+}
+
+bool network_get_force_default_owe_group(struct network *network)
+{
+	return network->force_default_owe_group;
 }
 
 static inline bool __bss_is_sae(const struct scan_bss *bss,
@@ -1154,7 +1170,7 @@ struct erp_cache_entry *network_get_erp_cache(struct network *network)
 }
 
 const struct l_queue_entry *network_bss_list_get_entries(
-						struct network *network)
+						const struct network *network)
 {
 	return l_queue_get_entries(network->bss_list);
 }
@@ -1523,6 +1539,7 @@ static struct l_dbus_message *network_connect_8021x(struct network *network,
 	r = eap_check_settings(network->settings, network->secrets, "EAP-",
 				true, &missing_secrets);
 	if (r) {
+#ifdef HAVE_DBUS
 		if (r == -EUNATCH)
 			reply = dbus_error_not_available(message);
 		else if (r == -ENOTSUP)
@@ -1531,6 +1548,7 @@ static struct l_dbus_message *network_connect_8021x(struct network *network,
 			reply = dbus_error_failed(message);
 		else
 			reply = dbus_error_not_configured(message);
+#endif
 
 		goto error;
 	}
@@ -1541,7 +1559,9 @@ static struct l_dbus_message *network_connect_8021x(struct network *network,
 
 	if (l_queue_isempty(missing_secrets)) {
 		if (!network_set_8021x_secrets(network)) {
+#ifdef HAVE_DBUS
 			reply = dbus_error_failed(message);
+#endif
 
 			goto error;
 		}
@@ -1555,7 +1575,9 @@ static struct l_dbus_message *network_connect_8021x(struct network *network,
 				eap_secret_done))
 		return NULL;
 
+#ifdef HAVE_DBUS
 	reply = dbus_error_no_agent(message);
+#endif
 
 error:
 	network_settings_close(network);
@@ -1773,6 +1795,7 @@ static bool network_property_get_known_network(struct l_dbus *dbus,
 
 bool network_register(struct network *network, const char *path)
 {
+#ifdef HAVE_DBUS
 	if (!l_dbus_object_add_interface(dbus_get_bus(), path,
 					IWD_NETWORK_INTERFACE, network)) {
 		l_info("Unable to register %s interface",
@@ -1784,6 +1807,7 @@ bool network_register(struct network *network, const char *path)
 					L_DBUS_INTERFACE_PROPERTIES, network))
 		l_info("Unable to register %s interface",
 						L_DBUS_INTERFACE_PROPERTIES);
+#endif
 
 	network->object_path = l_strdup(path);
 
@@ -1792,7 +1816,9 @@ bool network_register(struct network *network, const char *path)
 
 static void network_unregister(struct network *network, int reason)
 {
+#ifdef HAVE_DBUS
 	struct l_dbus *dbus = dbus_get_bus();
+#endif
 
 	if (network->connect_after_anqp)
 		dbus_pending_reply(&network->connect_after_anqp,
@@ -1805,7 +1831,9 @@ static void network_unregister(struct network *network, int reason)
 	agent_request_cancel(network->agent_request, reason);
 	network_settings_close(network);
 
+#ifdef HAVE_DBUS
 	l_dbus_unregister_object(dbus, network->object_path);
+#endif
 
 	l_free(network->object_path);
 	network->object_path = NULL;
@@ -2007,9 +2035,11 @@ static void event_watch_changed(enum station_event state,
 		l_debug("ANQP complete, resuming connect to %s", network->ssid);
 
 		if (!network_settings_load(network)) {
+#ifdef HAVE_DBUS
 			reply = dbus_error_not_configured(
 						network->connect_after_anqp);
 			dbus_pending_reply(&network->connect_after_anqp, reply);
+#endif
 			return;
 		}
 
@@ -2017,10 +2047,12 @@ static void event_watch_changed(enum station_event state,
 					network_bss_select(network, true),
 					network->connect_after_anqp);
 
+#ifdef HAVE_DBUS
 		if (reply)
 			l_dbus_send(dbus_get_bus(), reply);
 
 		l_dbus_message_unref(network->connect_after_anqp);
+#endif
 		network->connect_after_anqp = NULL;
 
 		break;
@@ -2069,10 +2101,12 @@ static void setup_network_interface(struct l_dbus_interface *interface)
 
 static int network_init(void)
 {
+#ifdef HAVE_DBUS
 	if (!l_dbus_register_interface(dbus_get_bus(), IWD_NETWORK_INTERFACE,
 					setup_network_interface, NULL, false))
 		l_error("Unable to register %s interface",
 						IWD_NETWORK_INTERFACE);
+#endif
 
 	known_networks_watch =
 		known_networks_watch_add(known_networks_changed, NULL, NULL);
@@ -2090,7 +2124,9 @@ static void network_exit(void)
 	station_remove_event_watch(event_watch);
 	event_watch = 0;
 
+#ifdef HAVE_DBUS
 	l_dbus_unregister_interface(dbus_get_bus(), IWD_NETWORK_INTERFACE);
+#endif
 }
 
 IWD_MODULE(network, network_init, network_exit)
