@@ -58,6 +58,8 @@ void network_sync_settings(struct network *network);
 
 const struct network_info *network_get_info(const struct network *network);
 void network_set_info(struct network *network, struct network_info *info);
+void network_set_force_default_owe_group(struct network *network);
+bool network_get_force_default_owe_group(struct network *network);
 
 int network_can_connect_bss(struct network *network,
 						const struct scan_bss *bss);
@@ -89,7 +91,7 @@ void network_blacklist_add(struct network *network, struct scan_bss *bss);
 struct erp_cache_entry *network_get_erp_cache(struct network *network);
 
 const struct l_queue_entry *network_bss_list_get_entries(
-						struct network *network);
+						const struct network *network);
 
 struct l_dbus_message *__network_connect(struct network *network,
 						struct scan_bss *bss,
